<?php

if (!defined('ABSPATH')) {
    die();
}

//Array access allows the object to be accessed as an array
class Translation implements ArrayAccess
{
    public static $api = null;
    public static $translationsFolder = ABSPATH . "translations/";
    public static $nextUpdateFile = null;
    
    protected static $googleDocsLink = "https://docs.google.com:443/spreadsheets/d/1vDfxEq-fZF0m6B8ClD6BaBqjy9oJOTeU44oOcl5O6wE/export?format=csv&id=1vDfxEq-fZF0m6B8ClD6BaBqjy9oJOTeU44oOcl5O6wE&gid=245350972";
    protected static $translations = array();
    
    public function createTranslations(){
        //Get the selected language
        $lang = self::$api->getWebsiteOption("WEBHOME_HEADER_LANGUAGE");
        $lang = $lang != null ? $lang : "SWEDISH";
        $filePath = self::getTranslationFilePath($lang);

        //Create the files if they don't exist
        if(!file_exists(self::getTranslationFilePath($lang)))
            self::createTranslationFiles();

        //Store the translations
        self::$translations = json_decode(file_get_contents($filePath), true);

        //If the file containing the next update date is missing it is recreated with the current time. This will force the files to be updated.
        if(!file_exists(self::$nextUpdateFile)){
            file_put_contents(self::$nextUpdateFile, date('d-m-Y', time()));
        }

        //If the langugage files must be updated, they are updated after the user has loaded the current language files. Also, the update is done via an ajax request on the backend.
        //This way the files can be updated without the user being affected with slow load times. The next time a page is visited, the new translation files will be used.
        if(strtotime(file_get_contents(self::$nextUpdateFile)) <= time()){    
            MSPECS::includeFile("includes/updateTranslations.phtml");
        }
    }

    /**
     * Creates the files that stores the translations
     */
    public static function createTranslationFiles(){
        if(!is_dir(self::$translationsFolder) && !mkdir(self::$translationsFolder)){
            throw new Exception("Could not create translation folder");
        }
        
        $translationsSheet = fopen(self::$googleDocsLink, "r");
        if(!$translationsSheet)
            throw new Exception("Could not download translations");
        
        $languages = self::getLanguages($translationsSheet);
        $parsedTranslations = self::parseTranslationSheet($translationsSheet);
        
        fclose($translationsSheet);

        self::saveTranslationFiles($parsedTranslations, $languages);

        //Create a file with the next update date
        $nextUpdateFileCreated = file_put_contents(
            self::$nextUpdateFile,
            date(
                'd-m-Y',
                strtotime("+1 day")
            )
        );

        if($nextUpdateFileCreated === false)
            throw new Exception("Could not create the next update file");
    }

    /**
     * Gets the languages specified in the first row. Must be called before parsing.
     * @param resource $translationsSheet The downloaded sheet that contains the translations
     * @return array An array with all the available languages
     */
    private static function getLanguages($translationsSheet){
        self::validateSheet($translationsSheet);
        $languageRow = fgetcsv($translationsSheet);

        //Remove the first cell since it is not a language
        $languages = array_splice($languageRow, 1);
        return $languages;
    }

    /**
     * Parses a sheet containing translations into an array
     * @param resource $translationsSheet The downloaded sheet that contains the translations
     * @return array An associative array with the translation constant as key and the translations as values
     */
    private static function parseTranslationSheet($translationsSheet){
        self::validateSheet($translationsSheet);

        $translations = array();
        //Use fgetcsv because it can handle multiline and empty values
        while (($row = fgetcsv($translationsSheet)) !== false){
            //Uses the constant as a key for the translations
            $cons = $row[0];
            $consTranslations = array_splice($row, 1);

            $translations = array_merge($translations, array($cons => $consTranslations));
        }
        
        return $translations;
    }

    /**
     * Validates if translationSheet is a resource
     * @param mixed $translationsSheet The translation sheet to be validated
     */
    private static function validateSheet($translationsSheet){
        if (is_resource($translationsSheet) === false) {
            throw new InvalidArgumentException(
                sprintf(
                    'Argument must be a valid resource type. %s given.',
                    gettype($translationsSheet)
                )
            );
        }
    }

    /**
     * Save the translations into seperate files for every language
     * @param array $translations An array consisting of all translations
     * @param array $languages An array containing all available languages
     */
    private static function saveTranslationFiles(array $translations, array $languages){
        //For every language, create a key-value paired array with the translations for the current language and save it into a json file
        foreach($languages as $index => $language){
            $currentLanguageTranslations = array();

            foreach($translations as $key => $translation){
                $currentLanguageTranslations[$key] = $translation[$index];
            }

            $translationFileCreated = file_put_contents(
                self::getTranslationFilePath($language),
                json_encode(
                    $currentLanguageTranslations,
                    JSON_PRETTY_PRINT
                )
            );

            if($translationFileCreated === false)
                throw new Exception("Could not create a translation file");
        }
    }

    protected static function getTranslationFilePath($language){
        return self::$translationsFolder . "locale-{$language}.json";
    }

    //Implementation of ArrayAccess abstract methods
    public function offsetExists($offset){
        return isset(self::$translations[$offset]);
    }

    //If the specified translation(offset) is not defined or it has no translation, it is returned as it was sent
    public function offsetGet($offset){
        return (isset(self::$translations[$offset]) && self::$translations[$offset] != null) ? self::$translations[$offset] : $offset;
    }

    public function offsetSet($offset, $value){
        if (is_null($offset)) {
            self::$translations[] = $value;
        } else {
            self::$translations[$offset] = $value;
        }
    }

    public function offsetUnset($offset){
        unset(self::$translations[$offset]);
    }
}

Translation::$api = MSPECS::getApi();
Translation::$nextUpdateFile = Translation::$translationsFolder . "nextUpdate.txt";
?>